"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createProductionBuild = void 0;
process.env.BABEL_ENV = 'production';
process.env.NODE_ENV = 'production';
// Makes the script crash on unhandled rejections instead of silently
// ignoring them. In the future, promise rejections that are not handled will
// terminate the Node.js process with a non-zero exit code.
process.on('unhandledRejection', (err) => {
    throw err;
});
const path_1 = __importDefault(require("path"));
const chalk_1 = __importDefault(require("react-dev-utils/chalk"));
const checkRequiredFiles_1 = __importDefault(require("react-dev-utils/checkRequiredFiles"));
const FileSizeReporter_1 = __importDefault(require("react-dev-utils/FileSizeReporter"));
const formatWebpackMessages_1 = __importDefault(require("react-dev-utils/formatWebpackMessages"));
const printBuildError_1 = __importDefault(require("react-dev-utils/printBuildError"));
const printHostingInstructions_1 = __importDefault(require("react-dev-utils/printHostingInstructions"));
const deepmerge_1 = __importDefault(require("deepmerge"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const webpack_1 = __importDefault(require("webpack"));
const env_1 = require("../config/env");
const paths_1 = __importDefault(require("../config/paths"));
const shared_1 = require("../config/shared");
const webpack_config_1 = __importDefault(require("../config/webpack.config"));
const { measureFileSizesBeforeBuild, printFileSizesAfterBuild } = FileSizeReporter_1.default;
// These sizes are pretty large. We'll warn for bundles exceeding them.
const WARN_AFTER_BUNDLE_GZIP_SIZE = 512 * 1024;
const WARN_AFTER_CHUNK_GZIP_SIZE = 1024 * 1024;
function copyPublicFolder(paths) {
    fs_extra_1.default.copySync(paths.appPublic, paths.appBuild, {
        dereference: true,
        filter: (file) => file !== paths.appHtml,
    });
}
// Generate configuration
// const config = configFactory('production', paths);
// Create the production build and print the deployment instructions.
function build(config, previousFileSizes) {
    console.log('Creating an optimized production build...');
    const compiler = (0, webpack_1.default)(config);
    return new Promise((resolve, reject) => {
        compiler.run((err, stats) => {
            let messages;
            if (err) {
                if (!err.message) {
                    return reject(err);
                }
                const errMessage = err.message;
                messages = (0, formatWebpackMessages_1.default)({
                    errors: [errMessage],
                    warnings: [],
                });
            }
            else {
                messages = (0, formatWebpackMessages_1.default)(stats === null || stats === void 0 ? void 0 : stats.toJson({ all: false, warnings: true, errors: true }));
            }
            if (messages.errors.length) {
                // Only keep the first error. Others are often indicative
                // of the same problem, but confuse the reader with noise.
                if (messages.errors.length > 1) {
                    messages.errors.length = 1;
                }
                return reject(new Error(messages.errors.join('\n\n')));
            }
            if (process.env.CI &&
                (typeof process.env.CI !== 'string' || process.env.CI.toLowerCase() !== 'false') &&
                messages.warnings.length) {
                console.log(chalk_1.default.yellow('\nTreating warnings as errors because process.env.CI = true.\n' +
                    'Most CI servers set it automatically.\n'));
                return reject(new Error(messages.warnings.join('\n\n')));
            }
            const resolveArgs = {
                stats,
                previousFileSizes,
                warnings: messages.warnings,
            };
            return resolve(resolveArgs);
        });
    });
}
async function createProductionBuild(paths, userConfig = shared_1.defaultDevConfig) {
    var _a;
    // Warn and crash if required files are missing
    if (!(0, checkRequiredFiles_1.default)([paths.appHtml, paths.appIndexJs])) {
        process.exit(1);
    }
    const useCustomConfig = fs_extra_1.default.existsSync(paths.appCustomConfig);
    const { configureWebpack, ...options } = useCustomConfig
        ? deepmerge_1.default.all([shared_1.defaultDevConfig, userConfig, (_a = require(paths.appCustomConfig)) !== null && _a !== void 0 ? _a : {}])
        : Object.assign(shared_1.defaultDevConfig, userConfig);
    const baseConfig = (0, webpack_config_1.default)('production', paths, options);
    const config = configureWebpack && typeof configureWebpack === 'function'
        ? configureWebpack(baseConfig, 'production')
        : baseConfig;
    const previousSizes = await measureFileSizesBeforeBuild(paths.appBuild);
    try {
        // Remove all content but keep the directory so that
        // if you're in it, you don't end up in Trash
        fs_extra_1.default.emptyDirSync(paths.appBuild);
        // Merge with the public folder
        copyPublicFolder(paths);
        if (useCustomConfig) {
            console.log(`${chalk_1.default.cyan('Using custom config from:')} ${chalk_1.default.yellow(paths.appCustomConfig)}\n`);
        }
        // Start the webpack build
        const { stats, previousFileSizes, warnings } = (await build(config, previousSizes));
        if (warnings.length) {
            console.log(chalk_1.default.yellow('Compiled with warnings.\n'));
            console.log(warnings.join('\n\n'));
            console.log(`\nSearch for the ${chalk_1.default.underline(chalk_1.default.yellow('keywords'))}  to learn more about each warning.`);
            console.log(`To ignore, add ${chalk_1.default.cyan('// eslint-disable-next-line')} to the line before.\n`);
        }
        else {
            console.log(chalk_1.default.green('Compiled successfully.\n'));
        }
        console.log('File sizes after gzip:\n');
        printFileSizesAfterBuild(stats, previousFileSizes, paths.appBuild, WARN_AFTER_BUNDLE_GZIP_SIZE, WARN_AFTER_CHUNK_GZIP_SIZE);
        console.log();
        const useYarn = fs_extra_1.default.existsSync(paths.yarnLockFile);
        const appPackage = require(paths.appPackageJson);
        const publicUrl = paths.publicUrlOrPath;
        const { publicPath } = config.output;
        const buildFolder = path_1.default.relative(process.cwd(), paths.appBuild);
        (0, printHostingInstructions_1.default)(appPackage, publicUrl, publicPath, buildFolder, useYarn);
    }
    catch (err) {
        const tscCompileOnError = process.env.TSC_COMPILE_ON_ERROR === 'true';
        if (tscCompileOnError) {
            console.log(chalk_1.default.yellow('Compiled with the following type errors (you may want to check these before deploying your app):\n'));
            (0, printBuildError_1.default)(err);
        }
        else {
            console.log(chalk_1.default.red('Failed to compile.\n'));
            (0, printBuildError_1.default)(err);
            process.exit(1);
        }
    }
}
exports.createProductionBuild = createProductionBuild;
if (require.main === module) {
    (0, env_1.getEnviroment)(paths_1.default);
    createProductionBuild(paths_1.default);
}
//# sourceMappingURL=build.js.map