"use strict";
/* eslint-disable no-use-before-define, no-param-reassign */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createTemplate = void 0;
const os_1 = __importDefault(require("os"));
const path_1 = __importDefault(require("path"));
const chalk_1 = __importDefault(require("chalk"));
const cross_spawn_1 = __importDefault(require("cross-spawn"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const helper_1 = require("./helper");
async function createTemplate(projectName) {
    return getProjectName({ projectName })
        .then(createProjectFolder)
        .then(copyTemplate)
        .then(createPackageJson)
        .then(changeCurrentDirectory)
        .then(installDependencies)
        .then(startGitRepo)
        .then(successMessage);
}
exports.createTemplate = createTemplate;
async function getProjectName(config) {
    const { projectName } = config;
    const [result, reason] = (0, helper_1.validProjectName)(projectName);
    const isValidName = result && !!projectName;
    if (!isValidName) {
        if (projectName) {
            console.error(chalk_1.default.red(`Nome Invalido: ${reason}`));
        }
        const { inputName } = await (0, helper_1.projectNameInput)();
        config.projectName = inputName;
    }
    return config;
}
async function createProjectFolder(config) {
    const projectFolder = path_1.default.resolve(config.projectName);
    await fs_extra_1.default.ensureDir(projectFolder);
    config.projectFolder = projectFolder;
    return config;
}
async function copyTemplate(config) {
    const { projectFolder } = config;
    await fs_extra_1.default.copy(helper_1.templateFolder, projectFolder);
    console.log(chalk_1.default.green(`Criando novo projeto em:\n${chalk_1.default.cyan(projectFolder)}\n`));
    return config;
}
async function createPackageJson(config) {
    const { projectName, projectFolder } = config;
    await fs_extra_1.default.writeJson(path_1.default.join(projectFolder, 'package.json'), {
        name: projectName,
        version: '0.1.0',
        private: true,
        author: process.env.USER || os_1.default.hostname(),
        license: 'MIT',
        scripts: {
            start: 'scripts dev',
            build: 'scripts build',
            test: 'scripts test -- --config=jest.config.js',
            lint: 'eslint --quiet src --ext ts,tsx',
            'lint:full': 'eslint src --ext ts,tsx',
            'lint:fix': 'eslint --fix src --ext ts,tsx',
            'prettier:write': 'prettier src/ --write',
            'audit-dependencies': 'audit-ci --config audit-ci.json',
        },
        husky: {
            hooks: {
                'pre-commit': 'lint-staged',
                'commit-msg': 'commitlint -E HUSKY_GIT_PARAMS',
            },
        },
        'lint-staged': {
            '*.{json,css,md}': ['yarn prettier:write'],
            '*.{ts,tsx}': ['yarn audit-dependencies', 'yarn lint', 'yarn prettier:write'],
        },
    }, {
        spaces: 2,
    });
    return config;
}
async function changeCurrentDirectory(config) {
    process.chdir(config.projectFolder);
    return config;
}
async function installDependencies(config) {
    const useYarn = (0, helper_1.shouldUseYarn)();
    const displayCommand = useYarn ? 'yarn' : 'npm';
    console.log(`Instalando dependencias com ${chalk_1.default.yellow(useYarn ? 'yarn' : 'npm')}\n`);
    config.useYarn = useYarn;
    config.displayCommand = displayCommand;
    return new Promise((resolve, reject) => {
        let command = 'npm';
        let args = ['install', '--save', '--loglevel', 'error'].concat(helper_1.templateDependencies);
        if (useYarn) {
            command = 'yarnpkg';
            args = ['add', '--exact'].concat(helper_1.templateDependencies);
        }
        const child = (0, cross_spawn_1.default)(command, args, { stdio: 'inherit' });
        child.on('close', (code) => {
            if (code !== 0) {
                reject(new Error(`${command} ${args.join(' ')}`));
            }
            else {
                resolve(config);
            }
        });
    });
}
async function startGitRepo(config) {
    const success = (0, helper_1.tryGitInit)();
    config.gitStarted = success;
    if (success) {
        const { useYarn, projectFolder } = config;
        const hooks = ['lint-staged', 'husky', '@commitlint/cli', '@commitlint/config-conventional'];
        let command = 'npm';
        let args = ['install', '--save', '--loglevel', 'error'].concat(hooks);
        if (useYarn) {
            command = 'yarnpkg';
            args = ['add', '--dev', '--exact'].concat(hooks);
        }
        console.log(`Iniciando Git Hooks\n`);
        return new Promise((resolve, reject) => {
            const child = (0, cross_spawn_1.default)(command, args, { stdio: 'inherit' });
            child.on('close', (code) => {
                if (code !== 0) {
                    reject(new Error(`${command} ${args.join(' ')}`));
                }
                else {
                    config.gitInitialCommit = (0, helper_1.tryGitCommit)(projectFolder);
                    resolve(config);
                }
            });
        });
    }
    return config;
}
async function successMessage(config) {
    if (config.gitStarted && config.gitInitialCommit) {
        const { projectName, projectFolder, displayCommand, useYarn } = config;
        console.log();
        console.log(chalk_1.default.green(`Sucesso! Projeto ${chalk_1.default.white(projectName)} Criado em:\n${chalk_1.default.white(projectFolder)}`));
        console.log();
        console.log('Comandos:\n');
        console.log(chalk_1.default.cyan(`${displayCommand} start`));
        console.log(chalk_1.default.cyan(`${displayCommand} ${useYarn ? '' : 'run '}build`));
        console.log(`\n\n`);
    }
}
//# sourceMappingURL=index.js.map